@@ -687,7 +687,7 @@ void qtractorMidiEditTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		// New loop-start boundary...
 		if (pSession->editHead() < pSession->loopEnd()) {
 			// Yep, new loop-start point...
-			m_pEditor->commands()->exec(
+			m_pEditor->execute(
 				new qtractorSessionLoopCommand(pSession,
 					pSession->editHead(), pSession->loopEnd()));
 		}
@@ -696,7 +696,7 @@ void qtractorMidiEditTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		// New loop-end boundary...
 		if (pSession->loopStart() < pSession->editTail()) {
 			// Yep, new loop-end point...
-			m_pEditor->commands()->exec(
+			m_pEditor->execute(
 				new qtractorSessionLoopCommand(pSession,
 					pSession->loopStart(), pSession->editTail()));
 		}
@@ -705,7 +705,7 @@ void qtractorMidiEditTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		// New punch-in boundary...
 		if (pSession->editHead() < pSession->punchOut()) {
 			// Yep, new punch-in point...
-			m_pEditor->commands()->exec(
+			m_pEditor->execute(
 				new qtractorSessionPunchCommand(pSession,
 					pSession->editHead(), pSession->punchOut()));
 		}
@@ -714,7 +714,7 @@ void qtractorMidiEditTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		// New punch-out boundary...
 		if (pSession->punchIn() < pSession->editTail()) {
 			// Yep, new punch-out point...
-			m_pEditor->commands()->exec(
+			m_pEditor->execute(
 				new qtractorSessionPunchCommand(pSession,
 					pSession->punchIn(), pSession->editTail()));
 		}
