@@ -134,6 +134,7 @@ void qtractorOptions::loadOptions (void)
 
 	// Transport options group.
 	m_settings.beginGroup("/Transport");
+	bCountIn         = m_settings.value("/CountIn", false).toBool();
 	bMetronome       = m_settings.value("/Metronome", false).toBool();
 	bFollowPlayhead  = m_settings.value("/FollowPlayhead", true).toBool();
 	bAutoBackward    = m_settings.value("/AutoBackward", false).toBool();
@@ -159,6 +160,8 @@ void qtractorOptions::loadOptions (void)
 	bAudioPlayerBus      = m_settings.value("/PlayerBus", false).toBool();
 	bAudioMetroBus       = m_settings.value("/MetroBus", false).toBool();
 	bAudioMetronome      = m_settings.value("/Metronome", false).toBool();
+	iAudioCountInMode    = m_settings.value("/CountInMode", 0).toInt();
+	iAudioCountInBeats   = m_settings.value("/CountInBeats", 4).toInt();
 	bAudioMasterAutoConnect = m_settings.value("/MasterAutoConnect", true).toBool();
 	bAudioPlayerAutoConnect = m_settings.value("/PlayerAutoConnect", true).toBool();
 	bAudioMetroAutoConnect = m_settings.value("/MetroAutoConnect", true).toBool();
@@ -176,6 +179,8 @@ void qtractorOptions::loadOptions (void)
 	bMidiControlBus    = m_settings.value("/ControlBus", false).toBool();
 	bMidiMetroBus      = m_settings.value("/MetroBus", false).toBool();
 	bMidiMetronome     = m_settings.value("/Metronome", true).toBool();
+	iMidiCountInMode   = m_settings.value("/CountInMode", 0).toInt();
+	iMidiCountInBeats  = m_settings.value("/CountInBeats", 4).toInt();
 	iMidiMetroOffset   = m_settings.value("/MetroOffset", 0).toInt();
 	iMidiMmcDevice     = m_settings.value("/MmcDevice", 0x7f).toInt();
 	iMidiMmcMode       = m_settings.value("/MmcMode", 3).toInt();
@@ -444,6 +449,7 @@ void qtractorOptions::saveOptions (void)
 
 	// Transport options group.
 	m_settings.beginGroup("/Transport");
+	m_settings.setValue("/CountIn", bCountIn);
 	m_settings.setValue("/Metronome", bMetronome);
 	m_settings.setValue("/FollowPlayhead", bFollowPlayhead);
 	m_settings.setValue("/AutoBackward", bAutoBackward);
@@ -469,6 +475,8 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/PlayerBus", bAudioPlayerBus);
 	m_settings.setValue("/MetroBus", bAudioMetroBus);
 	m_settings.setValue("/Metronome", bAudioMetronome);
+	m_settings.setValue("/CountInMode", iAudioCountInMode);
+	m_settings.setValue("/CountInBeats", iAudioCountInBeats);
 	m_settings.setValue("/MasterAutoConnect", bAudioMasterAutoConnect);
 	m_settings.setValue("/PlayerAutoConnect", bAudioPlayerAutoConnect);
 	m_settings.setValue("/MetroAutoConnect", bAudioMetroAutoConnect);
@@ -486,6 +494,8 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/ControlBus", bMidiControlBus);
 	m_settings.setValue("/MetroBus", bMidiMetroBus);
 	m_settings.setValue("/Metronome", bMidiMetronome);
+	m_settings.setValue("/CountInMode", iMidiCountInMode);
+	m_settings.setValue("/CountInBeats", iMidiCountInBeats);
 	m_settings.setValue("/MetroOffset", iMidiMetroOffset);
 	m_settings.setValue("/MmcDevice", iMidiMmcDevice);
 	m_settings.setValue("/MmcMode", iMidiMmcMode);
