@@ -254,6 +254,8 @@ void qtractorSession::clear (void)
 
 	m_pCurrentTrack = nullptr;
 
+	m_pCommands->clear();
+
 	m_tracks.clear();
 	m_cursors.clear();
 
@@ -265,8 +267,6 @@ void qtractorSession::clear (void)
 	m_pMidiEngine->clear();
 	m_pAudioEngine->clear();
 
-	m_pCommands->clear();
-
 	m_pFiles->clear();
 
 	m_filePaths.clear();
@@ -1337,6 +1337,8 @@ void qtractorSession::setPlayHead ( unsigned long iPlayHead )
 	if (!bPlaying) {
 		// Update time(base)/BBT info...
 		m_pAudioEngine->updateTimeInfo(iPlayHead);
+		// Reset step-input tracking...
+		m_pMidiEngine->proxy()->notifyInpEvent(qtractorMidiEngine::InpReset);
 		// Sync all track automation...
 		process_curve(iPlayHead);
 	}
@@ -1377,6 +1379,8 @@ void qtractorSession::setPlayHeadEx ( unsigned long iPlayHead )
 	} else {
 		// Update time(base)/BBT info...
 		m_pAudioEngine->updateTimeInfo(iPlayHead);
+		// Reset step-input tracking...
+		m_pMidiEngine->proxy()->notifyInpEvent(qtractorMidiEngine::InpReset);
 		// Sync all track automation...
 		process_curve(iPlayHead);
 	}
@@ -1639,6 +1643,19 @@ QString qtractorSession::createFilePath (
 }
 
 
+// Session directory relative/absolute file path helpers.
+QString qtractorSession::relativeFilePath ( const QString& sFilename ) const
+{
+	return QDir(sessionDir()).relativeFilePath(sFilename);
+}
+
+
+QString qtractorSession::absoluteFilePath ( const QString& sFilename ) const
+{
+	return QDir::cleanPath(QDir(sessionDir()).absoluteFilePath(sFilename));
+}
+
+
 // Consolidated session record state.
 void qtractorSession::setRecording ( bool bRecording )
 {
@@ -2293,8 +2310,13 @@ bool qtractorSession::loadElement (
 							continue;
 						if (eItem.tagName() == "text")
 							sText = eItem.text();
-						else if (eItem.tagName() == "color")
-							rgbColor.setNamedColor(eItem.text());
+						else if (eItem.tagName() == "color") {
+						#if QT_VERSION >= QT_VERSION_CHECK(6, 6, 0)
+							rgbColor = QColor::fromString(eItem.text());
+						#else
+							rgbColor = QColor(eItem.text());
+						#endif
+						}
 						else if (eItem.tagName() == "accidentals")
 							iAccidentals = eItem.text().toInt();
 						else if (eItem.tagName() == "mode")
@@ -2627,7 +2649,7 @@ void qtractorSession::renameSession (
 				qtractorMidiClip *pMidiClip
 					= static_cast<qtractorMidiClip *> (pClip);
 				if (pMidiClip)
-					pMidiClip->setFilenameEx(sNewFilePath, false);
+					pMidiClip->setFilenameEx(sNewFilePath, true);
 			} else {
 				pClip->close();
 				pClip->setFilename(sNewFilePath);
@@ -2640,7 +2662,7 @@ void qtractorSession::renameSession (
 					= pFileListView->findFileItem(sOldFilePath);
 				if (pFileItem) {
 					pGroupItem = pFileItem->groupItem();
-					m_pFiles->removeFileItem(iFileType, pFileItem);
+					m_pFiles->removeFileItem(iFileType, pFileItem->path());
 					delete pFileItem;
 				}
 				pFileListView->addFileItem(sNewFilePath, pGroupItem);
