@@ -1,7 +1,7 @@
 // qtractorTrack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -1897,11 +1897,17 @@ bool qtractorTrack::loadElement (
 				if (eView.tagName() == "height") {
 					qtractorTrack::setHeight(eView.text().toInt());
 				} else if (eView.tagName() == "background-color") {
-					QColor bg; bg.setNamedColor(eView.text());
-					qtractorTrack::setBackground(bg);
+				#if QT_VERSION >= QT_VERSION_CHECK(6, 6, 0)
+					qtractorTrack::setBackground(QColor::fromString(eView.text()));
+				#else
+					qtractorTrack::setBackground(QColor(eView.text()));
+				#endif
 				} else if (eView.tagName() == "foreground-color") {
-					QColor fg; fg.setNamedColor(eView.text());
-					qtractorTrack::setForeground(fg);
+				#if QT_VERSION >= QT_VERSION_CHECK(6, 6, 0)
+					qtractorTrack::setForeground(QColor::fromString(eView.text()));
+				#else
+					qtractorTrack::setForeground(QColor(eView.text()));
+				#endif
 				}
 			}
 		}
@@ -2010,7 +2016,8 @@ bool qtractorTrack::saveElement (
 	pDocument->saveTextElement("solo",
 		qtractorDocument::textFromBool(qtractorTrack::isSolo()), &eState);
 	pDocument->saveTextElement("record",
-		qtractorDocument::textFromBool(qtractorTrack::isRecord()), &eState);
+		qtractorDocument::textFromBool(qtractorTrack::isRecord()
+			&& !qtractorTrack::isClipRecordEx()), &eState);
 	pDocument->saveTextElement("monitor",
 		qtractorDocument::textFromBool(qtractorTrack::isMonitor()), &eState);
 	pDocument->saveTextElement("gain",
